# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import user_get_params, user_list_params, user_create_params, user_update_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..types.user import User
from .._base_client import make_request_options
from ..types.user_list_response import UserListResponse

__all__ = ["UsersResource", "AsyncUsersResource"]


class UsersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> UsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return UsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> UsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return UsersResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str | None = None,
        role: str,
        user_id: str | Omit = omit,
        display_name: str | Omit = omit,
        email: str | Omit = omit,
        service_account: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> User:
        """Create User

        Args:
          role: The user's role, e.g.

        admin or user.

          user_id: The user ID to use in the user name. e.g. my-user If not specified, a default ID
              is generated from user.email.

          display_name: Human-readable display name of the user. e.g. "Alice" Must be fewer than 64
              characters long.

          email: The user's email address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/users"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users",
            body=maybe_transform(
                {
                    "role": role,
                    "display_name": display_name,
                    "email": email,
                    "service_account": service_account,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"user_id": user_id}, user_create_params.UserCreateParams),
            ),
            cast_to=User,
        )

    def update(
        self,
        user_id: str,
        *,
        account_id: str | None = None,
        role: str,
        display_name: str | Omit = omit,
        email: str | Omit = omit,
        service_account: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> User:
        """Update User

        Args:
          role: The user's role, e.g.

        admin or user.

          display_name: Human-readable display name of the user. e.g. "Alice" Must be fewer than 64
              characters long.

          email: The user's email address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._patch(
            f"/v1/accounts/{account_id}/users/{user_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}",
            body=maybe_transform(
                {
                    "role": role,
                    "display_name": display_name,
                    "email": email,
                    "service_account": service_account,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=User,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListResponse:
        """
        List Users

        Args:
          filter: Only users satisfying the provided filter (if specified) will be returned. See
              https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of users to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListUsers call. Provide this to retrieve
              the subsequent page. When paginating, all other parameters provided to ListUsers
              must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/users"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            cast_to=UserListResponse,
        )

    def get(
        self,
        user_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> User:
        """Get User

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/users/{user_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"read_mask": read_mask}, user_get_params.UserGetParams),
            ),
            cast_to=User,
        )


class AsyncUsersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncUsersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncUsersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncUsersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncUsersResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str | None = None,
        role: str,
        user_id: str | Omit = omit,
        display_name: str | Omit = omit,
        email: str | Omit = omit,
        service_account: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> User:
        """Create User

        Args:
          role: The user's role, e.g.

        admin or user.

          user_id: The user ID to use in the user name. e.g. my-user If not specified, a default ID
              is generated from user.email.

          display_name: Human-readable display name of the user. e.g. "Alice" Must be fewer than 64
              characters long.

          email: The user's email address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/users"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users",
            body=await async_maybe_transform(
                {
                    "role": role,
                    "display_name": display_name,
                    "email": email,
                    "service_account": service_account,
                },
                user_create_params.UserCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"user_id": user_id}, user_create_params.UserCreateParams),
            ),
            cast_to=User,
        )

    async def update(
        self,
        user_id: str,
        *,
        account_id: str | None = None,
        role: str,
        display_name: str | Omit = omit,
        email: str | Omit = omit,
        service_account: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> User:
        """Update User

        Args:
          role: The user's role, e.g.

        admin or user.

          display_name: Human-readable display name of the user. e.g. "Alice" Must be fewer than 64
              characters long.

          email: The user's email address.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._patch(
            f"/v1/accounts/{account_id}/users/{user_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}",
            body=await async_maybe_transform(
                {
                    "role": role,
                    "display_name": display_name,
                    "email": email,
                    "service_account": service_account,
                },
                user_update_params.UserUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=User,
        )

    async def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> UserListResponse:
        """
        List Users

        Args:
          filter: Only users satisfying the provided filter (if specified) will be returned. See
              https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "name".

          page_size: The maximum number of users to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListUsers call. Provide this to retrieve
              the subsequent page. When paginating, all other parameters provided to ListUsers
              must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/users"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    user_list_params.UserListParams,
                ),
            ),
            cast_to=UserListResponse,
        )

    async def get(
        self,
        user_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> User:
        """Get User

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/users/{user_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/users/{user_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"read_mask": read_mask}, user_get_params.UserGetParams),
            ),
            cast_to=User,
        )


class UsersResourceWithRawResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.create = to_raw_response_wrapper(
            users.create,
        )
        self.update = to_raw_response_wrapper(
            users.update,
        )
        self.list = to_raw_response_wrapper(
            users.list,
        )
        self.get = to_raw_response_wrapper(
            users.get,
        )


class AsyncUsersResourceWithRawResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.create = async_to_raw_response_wrapper(
            users.create,
        )
        self.update = async_to_raw_response_wrapper(
            users.update,
        )
        self.list = async_to_raw_response_wrapper(
            users.list,
        )
        self.get = async_to_raw_response_wrapper(
            users.get,
        )


class UsersResourceWithStreamingResponse:
    def __init__(self, users: UsersResource) -> None:
        self._users = users

        self.create = to_streamed_response_wrapper(
            users.create,
        )
        self.update = to_streamed_response_wrapper(
            users.update,
        )
        self.list = to_streamed_response_wrapper(
            users.list,
        )
        self.get = to_streamed_response_wrapper(
            users.get,
        )


class AsyncUsersResourceWithStreamingResponse:
    def __init__(self, users: AsyncUsersResource) -> None:
        self._users = users

        self.create = async_to_streamed_response_wrapper(
            users.create,
        )
        self.update = async_to_streamed_response_wrapper(
            users.update,
        )
        self.list = async_to_streamed_response_wrapper(
            users.list,
        )
        self.get = async_to_streamed_response_wrapper(
            users.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .shared.status import Status
from .shared.inference_parameters import InferenceParameters

__all__ = ["BatchInferenceJob", "JobProgress"]


class JobProgress(BaseModel):
    """Job progress."""

    cached_input_token_count: Optional[int] = FieldInfo(alias="cachedInputTokenCount", default=None)
    """The number of input tokens that hit the prompt cache."""

    epoch: Optional[int] = None
    """
    The epoch for which the progress percent is reported, usually starting from 0.
    This is optional for jobs that don't run in an epoch fasion, e.g. BIJ, EVJ.
    """

    failed_requests: Optional[int] = FieldInfo(alias="failedRequests", default=None)
    """Number of requests that failed to process."""

    input_tokens: Optional[int] = FieldInfo(alias="inputTokens", default=None)
    """Total number of input tokens processed."""

    output_rows: Optional[int] = FieldInfo(alias="outputRows", default=None)
    """Number of output rows generated."""

    output_tokens: Optional[int] = FieldInfo(alias="outputTokens", default=None)
    """Total number of output tokens generated."""

    percent: Optional[int] = None
    """Progress percent, within the range from 0 to 100."""

    successfully_processed_requests: Optional[int] = FieldInfo(alias="successfullyProcessedRequests", default=None)
    """Number of requests that were processed successfully."""

    total_input_requests: Optional[int] = FieldInfo(alias="totalInputRequests", default=None)
    """Total number of input requests/rows in the job."""

    total_processed_requests: Optional[int] = FieldInfo(alias="totalProcessedRequests", default=None)
    """Total number of requests that have been processed (successfully or failed)."""


class BatchInferenceJob(BaseModel):
    continued_from_job_name: Optional[str] = FieldInfo(alias="continuedFromJobName", default=None)
    """
    The resource name of the batch inference job that this job continues from. Used
    for lineage tracking to understand job continuation chains.
    """

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """The email address of the user who initiated this batch inference job."""

    create_time: Optional[datetime] = FieldInfo(alias="createTime", default=None)
    """The creation time of the batch inference job."""

    display_name: Optional[str] = FieldInfo(alias="displayName", default=None)

    inference_parameters: Optional[InferenceParameters] = FieldInfo(alias="inferenceParameters", default=None)
    """Parameters controlling the inference process."""

    input_dataset_id: Optional[str] = FieldInfo(alias="inputDatasetId", default=None)
    """The name of the dataset used for inference.

    This is required, except when continued_from_job_name is specified.
    """

    job_progress: Optional[JobProgress] = FieldInfo(alias="jobProgress", default=None)
    """Job progress."""

    model: Optional[str] = None
    """The name of the model to use for inference.

    This is required, except when continued_from_job_name is specified.
    """

    name: Optional[str] = None

    output_dataset_id: Optional[str] = FieldInfo(alias="outputDatasetId", default=None)
    """The name of the dataset used for storing the results.

    This will also contain the error file.
    """

    precision: Optional[
        Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
    ] = None
    """
    The precision with which the model should be served. If PRECISION_UNSPECIFIED, a
    default will be chosen based on the model.
    """

    state: Optional[
        Literal[
            "JOB_STATE_UNSPECIFIED",
            "JOB_STATE_CREATING",
            "JOB_STATE_RUNNING",
            "JOB_STATE_COMPLETED",
            "JOB_STATE_FAILED",
            "JOB_STATE_CANCELLED",
            "JOB_STATE_DELETING",
            "JOB_STATE_WRITING_RESULTS",
            "JOB_STATE_VALIDATING",
            "JOB_STATE_DELETING_CLEANING_UP",
            "JOB_STATE_PENDING",
            "JOB_STATE_EXPIRED",
            "JOB_STATE_RE_QUEUEING",
            "JOB_STATE_CREATING_INPUT_DATASET",
            "JOB_STATE_IDLE",
            "JOB_STATE_CANCELLING",
            "JOB_STATE_EARLY_STOPPED",
            "JOB_STATE_PAUSED",
        ]
    ] = None
    """JobState represents the state an asynchronous job can be in."""

    status: Optional[Status] = None

    update_time: Optional[datetime] = FieldInfo(alias="updateTime", default=None)
    """The update time for the batch inference job."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..types import account_get_params, account_list_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.account import Account
from ..types.account_list_response import AccountListResponse

__all__ = ["AccountsResource", "AsyncAccountsResource"]


class AccountsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AccountsResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListResponse:
        """
        List Accounts

        Args:
          filter: Only accounts satisfying the provided filter (if specified) will be returned.
              See https://google.aip.dev/160 for the filter grammar.

          order_by: Not supported. Accounts will be returned ordered by `name`.

          page_size: The maximum number of accounts to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListAccounts call. Provide this to
              retrieve the subsequent page. When paginating, all other parameters provided to
              ListAccounts must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/accounts" if self._client._base_url_overridden else "https://api.fireworks.ai/v1/accounts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            cast_to=AccountListResponse,
        )

    def get(
        self,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Account:
        """Get Account

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"read_mask": read_mask}, account_get_params.AccountGetParams),
            ),
            cast_to=Account,
        )


class AsyncAccountsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncAccountsResourceWithStreamingResponse(self)

    async def list(
        self,
        *,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccountListResponse:
        """
        List Accounts

        Args:
          filter: Only accounts satisfying the provided filter (if specified) will be returned.
              See https://google.aip.dev/160 for the filter grammar.

          order_by: Not supported. Accounts will be returned ordered by `name`.

          page_size: The maximum number of accounts to return. The maximum page_size is 200, values
              above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListAccounts call. Provide this to
              retrieve the subsequent page. When paginating, all other parameters provided to
              ListAccounts must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/accounts" if self._client._base_url_overridden else "https://api.fireworks.ai/v1/accounts",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    account_list_params.AccountListParams,
                ),
            ),
            cast_to=AccountListResponse,
        )

    async def get(
        self,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Account:
        """Get Account

        Args:
          read_mask: The fields to be returned in the response.

        If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"read_mask": read_mask}, account_get_params.AccountGetParams),
            ),
            cast_to=Account,
        )


class AccountsResourceWithRawResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.list = to_raw_response_wrapper(
            accounts.list,
        )
        self.get = to_raw_response_wrapper(
            accounts.get,
        )


class AsyncAccountsResourceWithRawResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.list = async_to_raw_response_wrapper(
            accounts.list,
        )
        self.get = async_to_raw_response_wrapper(
            accounts.get,
        )


class AccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AccountsResource) -> None:
        self._accounts = accounts

        self.list = to_streamed_response_wrapper(
            accounts.list,
        )
        self.get = to_streamed_response_wrapper(
            accounts.get,
        )


class AsyncAccountsResourceWithStreamingResponse:
    def __init__(self, accounts: AsyncAccountsResource) -> None:
        self._accounts = accounts

        self.list = async_to_streamed_response_wrapper(
            accounts.list,
        )
        self.get = async_to_streamed_response_wrapper(
            accounts.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo
from .splitted_param import SplittedParam
from .transformed_param import TransformedParam
from .evaluation_result_param import EvaluationResultParam

__all__ = ["DatasetUpdateParams"]


class DatasetUpdateParams(TypedDict, total=False):
    account_id: str

    display_name: Annotated[str, PropertyInfo(alias="displayName")]

    eval_protocol: Annotated[object, PropertyInfo(alias="evalProtocol")]

    evaluation_result: Annotated[EvaluationResultParam, PropertyInfo(alias="evaluationResult")]

    example_count: Annotated[str, PropertyInfo(alias="exampleCount")]

    external_url: Annotated[str, PropertyInfo(alias="externalUrl")]

    format: Literal["FORMAT_UNSPECIFIED", "CHAT", "COMPLETION", "RL"]

    source_job_name: Annotated[str, PropertyInfo(alias="sourceJobName")]
    """
    The resource name of the job that created this dataset (e.g., batch inference
    job). Used for lineage tracking to understand dataset provenance.
    """

    splitted: SplittedParam

    transformed: TransformedParam

    user_uploaded: Annotated[object, PropertyInfo(alias="userUploaded")]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._version import __version__
from .resources import (
    lora,
    users,
    models,
    secrets,
    accounts,
    api_keys,
    datasets,
    dpo_jobs,
    evaluators,
    completions,
    deployments,
    evaluation_jobs,
    batch_inference_jobs,
    deployment_shape_versions,
    supervised_fine_tuning_jobs,
    reinforcement_fine_tuning_jobs,
    reinforcement_fine_tuning_steps,
)
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import APIStatusError, FireworksError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)
from .resources.chat import chat

__all__ = [
    "Timeout",
    "Transport",
    "ProxiesTypes",
    "RequestOptions",
    "Fireworks",
    "AsyncFireworks",
    "Client",
    "AsyncClient",
]


class Fireworks(SyncAPIClient):
    chat: chat.ChatResource
    completions: completions.CompletionsResource
    batch_inference_jobs: batch_inference_jobs.BatchInferenceJobsResource
    deployments: deployments.DeploymentsResource
    models: models.ModelsResource
    lora: lora.LoraResource
    deployment_shape_versions: deployment_shape_versions.DeploymentShapeVersionsResource
    datasets: datasets.DatasetsResource
    supervised_fine_tuning_jobs: supervised_fine_tuning_jobs.SupervisedFineTuningJobsResource
    reinforcement_fine_tuning_jobs: reinforcement_fine_tuning_jobs.ReinforcementFineTuningJobsResource
    reinforcement_fine_tuning_steps: reinforcement_fine_tuning_steps.ReinforcementFineTuningStepsResource
    dpo_jobs: dpo_jobs.DpoJobsResource
    evaluation_jobs: evaluation_jobs.EvaluationJobsResource
    evaluators: evaluators.EvaluatorsResource
    accounts: accounts.AccountsResource
    users: users.UsersResource
    api_keys: api_keys.APIKeysResource
    secrets: secrets.SecretsResource
    with_raw_response: FireworksWithRawResponse
    with_streaming_response: FireworksWithStreamedResponse

    # client options
    api_key: str
    account_id: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Fireworks client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `FIREWORKS_API_KEY`
        - `account_id` from `FIREWORKS_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("FIREWORKS_API_KEY")
        if api_key is None:
            raise FireworksError(
                "The api_key client option must be set either by passing api_key to the client or by setting the FIREWORKS_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("FIREWORKS_ACCOUNT_ID")
        self.account_id = account_id

        if base_url is None:
            base_url = os.environ.get("FIREWORKS_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.fireworks.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = Stream

        self.chat = chat.ChatResource(self)
        self.completions = completions.CompletionsResource(self)
        self.batch_inference_jobs = batch_inference_jobs.BatchInferenceJobsResource(self)
        self.deployments = deployments.DeploymentsResource(self)
        self.models = models.ModelsResource(self)
        self.lora = lora.LoraResource(self)
        self.deployment_shape_versions = deployment_shape_versions.DeploymentShapeVersionsResource(self)
        self.datasets = datasets.DatasetsResource(self)
        self.supervised_fine_tuning_jobs = supervised_fine_tuning_jobs.SupervisedFineTuningJobsResource(self)
        self.reinforcement_fine_tuning_jobs = reinforcement_fine_tuning_jobs.ReinforcementFineTuningJobsResource(self)
        self.reinforcement_fine_tuning_steps = reinforcement_fine_tuning_steps.ReinforcementFineTuningStepsResource(
            self
        )
        self.dpo_jobs = dpo_jobs.DpoJobsResource(self)
        self.evaluation_jobs = evaluation_jobs.EvaluationJobsResource(self)
        self.evaluators = evaluators.EvaluatorsResource(self)
        self.accounts = accounts.AccountsResource(self)
        self.users = users.UsersResource(self)
        self.api_keys = api_keys.APIKeysResource(self)
        self.secrets = secrets.SecretsResource(self)
        self.with_raw_response = FireworksWithRawResponse(self)
        self.with_streaming_response = FireworksWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_account_id_path_param(self) -> str:
        from_client = self.account_id
        if from_client is not None:
            return from_client

        raise ValueError(
            "Missing account_id argument; Please provide it at the client level, e.g. Fireworks(account_id='abcd') or per method."
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncFireworks(AsyncAPIClient):
    chat: chat.AsyncChatResource
    completions: completions.AsyncCompletionsResource
    batch_inference_jobs: batch_inference_jobs.AsyncBatchInferenceJobsResource
    deployments: deployments.AsyncDeploymentsResource
    models: models.AsyncModelsResource
    lora: lora.AsyncLoraResource
    deployment_shape_versions: deployment_shape_versions.AsyncDeploymentShapeVersionsResource
    datasets: datasets.AsyncDatasetsResource
    supervised_fine_tuning_jobs: supervised_fine_tuning_jobs.AsyncSupervisedFineTuningJobsResource
    reinforcement_fine_tuning_jobs: reinforcement_fine_tuning_jobs.AsyncReinforcementFineTuningJobsResource
    reinforcement_fine_tuning_steps: reinforcement_fine_tuning_steps.AsyncReinforcementFineTuningStepsResource
    dpo_jobs: dpo_jobs.AsyncDpoJobsResource
    evaluation_jobs: evaluation_jobs.AsyncEvaluationJobsResource
    evaluators: evaluators.AsyncEvaluatorsResource
    accounts: accounts.AsyncAccountsResource
    users: users.AsyncUsersResource
    api_keys: api_keys.AsyncAPIKeysResource
    secrets: secrets.AsyncSecretsResource
    with_raw_response: AsyncFireworksWithRawResponse
    with_streaming_response: AsyncFireworksWithStreamedResponse

    # client options
    api_key: str
    account_id: str | None

    def __init__(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncFireworks client instance.

        This automatically infers the following arguments from their corresponding environment variables if they are not provided:
        - `api_key` from `FIREWORKS_API_KEY`
        - `account_id` from `FIREWORKS_ACCOUNT_ID`
        """
        if api_key is None:
            api_key = os.environ.get("FIREWORKS_API_KEY")
        if api_key is None:
            raise FireworksError(
                "The api_key client option must be set either by passing api_key to the client or by setting the FIREWORKS_API_KEY environment variable"
            )
        self.api_key = api_key

        if account_id is None:
            account_id = os.environ.get("FIREWORKS_ACCOUNT_ID")
        self.account_id = account_id

        if base_url is None:
            base_url = os.environ.get("FIREWORKS_BASE_URL")
        self._base_url_overridden = base_url is not None
        if base_url is None:
            base_url = f"https://api.fireworks.ai"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

        self._default_stream_cls = AsyncStream

        self.chat = chat.AsyncChatResource(self)
        self.completions = completions.AsyncCompletionsResource(self)
        self.batch_inference_jobs = batch_inference_jobs.AsyncBatchInferenceJobsResource(self)
        self.deployments = deployments.AsyncDeploymentsResource(self)
        self.models = models.AsyncModelsResource(self)
        self.lora = lora.AsyncLoraResource(self)
        self.deployment_shape_versions = deployment_shape_versions.AsyncDeploymentShapeVersionsResource(self)
        self.datasets = datasets.AsyncDatasetsResource(self)
        self.supervised_fine_tuning_jobs = supervised_fine_tuning_jobs.AsyncSupervisedFineTuningJobsResource(self)
        self.reinforcement_fine_tuning_jobs = reinforcement_fine_tuning_jobs.AsyncReinforcementFineTuningJobsResource(
            self
        )
        self.reinforcement_fine_tuning_steps = (
            reinforcement_fine_tuning_steps.AsyncReinforcementFineTuningStepsResource(self)
        )
        self.dpo_jobs = dpo_jobs.AsyncDpoJobsResource(self)
        self.evaluation_jobs = evaluation_jobs.AsyncEvaluationJobsResource(self)
        self.evaluators = evaluators.AsyncEvaluatorsResource(self)
        self.accounts = accounts.AsyncAccountsResource(self)
        self.users = users.AsyncUsersResource(self)
        self.api_keys = api_keys.AsyncAPIKeysResource(self)
        self.secrets = secrets.AsyncSecretsResource(self)
        self.with_raw_response = AsyncFireworksWithRawResponse(self)
        self.with_streaming_response = AsyncFireworksWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        account_id: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        client = self.__class__(
            api_key=api_key or self.api_key,
            account_id=account_id or self.account_id,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )
        client._base_url_overridden = self._base_url_overridden or base_url is not None
        return client

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    def _get_account_id_path_param(self) -> str:
        from_client = self.account_id
        if from_client is not None:
            return from_client

        raise ValueError(
            "Missing account_id argument; Please provide it at the client level, e.g. AsyncFireworks(account_id='abcd') or per method."
        )

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class FireworksWithRawResponse:
    def __init__(self, client: Fireworks) -> None:
        self.chat = chat.ChatResourceWithRawResponse(client.chat)
        self.completions = completions.CompletionsResourceWithRawResponse(client.completions)
        self.batch_inference_jobs = batch_inference_jobs.BatchInferenceJobsResourceWithRawResponse(
            client.batch_inference_jobs
        )
        self.deployments = deployments.DeploymentsResourceWithRawResponse(client.deployments)
        self.models = models.ModelsResourceWithRawResponse(client.models)
        self.lora = lora.LoraResourceWithRawResponse(client.lora)
        self.deployment_shape_versions = deployment_shape_versions.DeploymentShapeVersionsResourceWithRawResponse(
            client.deployment_shape_versions
        )
        self.datasets = datasets.DatasetsResourceWithRawResponse(client.datasets)
        self.supervised_fine_tuning_jobs = supervised_fine_tuning_jobs.SupervisedFineTuningJobsResourceWithRawResponse(
            client.supervised_fine_tuning_jobs
        )
        self.reinforcement_fine_tuning_jobs = (
            reinforcement_fine_tuning_jobs.ReinforcementFineTuningJobsResourceWithRawResponse(
                client.reinforcement_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_steps = (
            reinforcement_fine_tuning_steps.ReinforcementFineTuningStepsResourceWithRawResponse(
                client.reinforcement_fine_tuning_steps
            )
        )
        self.dpo_jobs = dpo_jobs.DpoJobsResourceWithRawResponse(client.dpo_jobs)
        self.evaluation_jobs = evaluation_jobs.EvaluationJobsResourceWithRawResponse(client.evaluation_jobs)
        self.evaluators = evaluators.EvaluatorsResourceWithRawResponse(client.evaluators)
        self.accounts = accounts.AccountsResourceWithRawResponse(client.accounts)
        self.users = users.UsersResourceWithRawResponse(client.users)
        self.api_keys = api_keys.APIKeysResourceWithRawResponse(client.api_keys)
        self.secrets = secrets.SecretsResourceWithRawResponse(client.secrets)


class AsyncFireworksWithRawResponse:
    def __init__(self, client: AsyncFireworks) -> None:
        self.chat = chat.AsyncChatResourceWithRawResponse(client.chat)
        self.completions = completions.AsyncCompletionsResourceWithRawResponse(client.completions)
        self.batch_inference_jobs = batch_inference_jobs.AsyncBatchInferenceJobsResourceWithRawResponse(
            client.batch_inference_jobs
        )
        self.deployments = deployments.AsyncDeploymentsResourceWithRawResponse(client.deployments)
        self.models = models.AsyncModelsResourceWithRawResponse(client.models)
        self.lora = lora.AsyncLoraResourceWithRawResponse(client.lora)
        self.deployment_shape_versions = deployment_shape_versions.AsyncDeploymentShapeVersionsResourceWithRawResponse(
            client.deployment_shape_versions
        )
        self.datasets = datasets.AsyncDatasetsResourceWithRawResponse(client.datasets)
        self.supervised_fine_tuning_jobs = (
            supervised_fine_tuning_jobs.AsyncSupervisedFineTuningJobsResourceWithRawResponse(
                client.supervised_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_jobs = (
            reinforcement_fine_tuning_jobs.AsyncReinforcementFineTuningJobsResourceWithRawResponse(
                client.reinforcement_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_steps = (
            reinforcement_fine_tuning_steps.AsyncReinforcementFineTuningStepsResourceWithRawResponse(
                client.reinforcement_fine_tuning_steps
            )
        )
        self.dpo_jobs = dpo_jobs.AsyncDpoJobsResourceWithRawResponse(client.dpo_jobs)
        self.evaluation_jobs = evaluation_jobs.AsyncEvaluationJobsResourceWithRawResponse(client.evaluation_jobs)
        self.evaluators = evaluators.AsyncEvaluatorsResourceWithRawResponse(client.evaluators)
        self.accounts = accounts.AsyncAccountsResourceWithRawResponse(client.accounts)
        self.users = users.AsyncUsersResourceWithRawResponse(client.users)
        self.api_keys = api_keys.AsyncAPIKeysResourceWithRawResponse(client.api_keys)
        self.secrets = secrets.AsyncSecretsResourceWithRawResponse(client.secrets)


class FireworksWithStreamedResponse:
    def __init__(self, client: Fireworks) -> None:
        self.chat = chat.ChatResourceWithStreamingResponse(client.chat)
        self.completions = completions.CompletionsResourceWithStreamingResponse(client.completions)
        self.batch_inference_jobs = batch_inference_jobs.BatchInferenceJobsResourceWithStreamingResponse(
            client.batch_inference_jobs
        )
        self.deployments = deployments.DeploymentsResourceWithStreamingResponse(client.deployments)
        self.models = models.ModelsResourceWithStreamingResponse(client.models)
        self.lora = lora.LoraResourceWithStreamingResponse(client.lora)
        self.deployment_shape_versions = deployment_shape_versions.DeploymentShapeVersionsResourceWithStreamingResponse(
            client.deployment_shape_versions
        )
        self.datasets = datasets.DatasetsResourceWithStreamingResponse(client.datasets)
        self.supervised_fine_tuning_jobs = (
            supervised_fine_tuning_jobs.SupervisedFineTuningJobsResourceWithStreamingResponse(
                client.supervised_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_jobs = (
            reinforcement_fine_tuning_jobs.ReinforcementFineTuningJobsResourceWithStreamingResponse(
                client.reinforcement_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_steps = (
            reinforcement_fine_tuning_steps.ReinforcementFineTuningStepsResourceWithStreamingResponse(
                client.reinforcement_fine_tuning_steps
            )
        )
        self.dpo_jobs = dpo_jobs.DpoJobsResourceWithStreamingResponse(client.dpo_jobs)
        self.evaluation_jobs = evaluation_jobs.EvaluationJobsResourceWithStreamingResponse(client.evaluation_jobs)
        self.evaluators = evaluators.EvaluatorsResourceWithStreamingResponse(client.evaluators)
        self.accounts = accounts.AccountsResourceWithStreamingResponse(client.accounts)
        self.users = users.UsersResourceWithStreamingResponse(client.users)
        self.api_keys = api_keys.APIKeysResourceWithStreamingResponse(client.api_keys)
        self.secrets = secrets.SecretsResourceWithStreamingResponse(client.secrets)


class AsyncFireworksWithStreamedResponse:
    def __init__(self, client: AsyncFireworks) -> None:
        self.chat = chat.AsyncChatResourceWithStreamingResponse(client.chat)
        self.completions = completions.AsyncCompletionsResourceWithStreamingResponse(client.completions)
        self.batch_inference_jobs = batch_inference_jobs.AsyncBatchInferenceJobsResourceWithStreamingResponse(
            client.batch_inference_jobs
        )
        self.deployments = deployments.AsyncDeploymentsResourceWithStreamingResponse(client.deployments)
        self.models = models.AsyncModelsResourceWithStreamingResponse(client.models)
        self.lora = lora.AsyncLoraResourceWithStreamingResponse(client.lora)
        self.deployment_shape_versions = (
            deployment_shape_versions.AsyncDeploymentShapeVersionsResourceWithStreamingResponse(
                client.deployment_shape_versions
            )
        )
        self.datasets = datasets.AsyncDatasetsResourceWithStreamingResponse(client.datasets)
        self.supervised_fine_tuning_jobs = (
            supervised_fine_tuning_jobs.AsyncSupervisedFineTuningJobsResourceWithStreamingResponse(
                client.supervised_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_jobs = (
            reinforcement_fine_tuning_jobs.AsyncReinforcementFineTuningJobsResourceWithStreamingResponse(
                client.reinforcement_fine_tuning_jobs
            )
        )
        self.reinforcement_fine_tuning_steps = (
            reinforcement_fine_tuning_steps.AsyncReinforcementFineTuningStepsResourceWithStreamingResponse(
                client.reinforcement_fine_tuning_steps
            )
        )
        self.dpo_jobs = dpo_jobs.AsyncDpoJobsResourceWithStreamingResponse(client.dpo_jobs)
        self.evaluation_jobs = evaluation_jobs.AsyncEvaluationJobsResourceWithStreamingResponse(client.evaluation_jobs)
        self.evaluators = evaluators.AsyncEvaluatorsResourceWithStreamingResponse(client.evaluators)
        self.accounts = accounts.AsyncAccountsResourceWithStreamingResponse(client.accounts)
        self.users = users.AsyncUsersResourceWithStreamingResponse(client.users)
        self.api_keys = api_keys.AsyncAPIKeysResourceWithStreamingResponse(client.api_keys)
        self.secrets = secrets.AsyncSecretsResourceWithStreamingResponse(client.secrets)


Client = Fireworks

AsyncClient = AsyncFireworks



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..shared_params.chat_message import ChatMessage
from ..shared_params.chat_completion_tool import ChatCompletionTool

__all__ = [
    "CompletionCreateParamsBase",
    "FunctionCall",
    "FunctionCallFunctionNameSpec",
    "Function",
    "Prediction",
    "PredictionPredictedOutput",
    "PredictionPredictedOutputContentUnionMember1",
    "PredictionPredictedOutputContentUnionMember1ImageURL",
    "ResponseFormat",
    "ToolChoice",
    "ToolChoiceFunctionSelection",
    "ToolChoiceFunctionSelectionFunction",
    "CompletionCreateParamsNonStreaming",
    "CompletionCreateParamsStreaming",
]


class CompletionCreateParamsBase(TypedDict, total=False):
    messages: Required[Iterable[ChatMessage]]
    """A list of messages comprising the conversation so far."""

    model: Required[str]
    """The name of the model to use.

    Example: `"accounts/fireworks/models/kimi-k2-instruct-0905"`
    """

    context_length_exceeded_behavior: Literal["error", "truncate"]
    """
    What to do if the token count of prompt plus `max_tokens` exceeds the model's
    context window.

    Passing `truncate` limits the `max_tokens` to at most
    `context_window_length - prompt_length`. This is the default.

    Passing `error` would trigger a request error.

    The default of `'truncate'` is selected as it allows to ask for high
    `max_tokens` value while respecting the context window length without having to
    do client-side prompt tokenization.

    Note, that it differs from OpenAI's behavior that matches that of `error`.
    """

    echo: Optional[bool]
    """Echo back the prompt in addition to the completion."""

    echo_last: Optional[int]
    """Echo back the last N tokens of the prompt in addition to the completion.

    This is useful for obtaining logprobs of the prompt suffix but without
    transferring too much data. Passing `echo_last=len(prompt)` is the same as
    `echo=True`
    """

    frequency_penalty: Optional[float]
    """Number between -2.0 and 2.0.

    Positive values penalize new tokens based on their existing frequency in the
    text so far, decreasing the model's likelihood to repeat the same line verbatim.

    Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
    samples somewhat. If the aim is to strongly suppress repetition, then one can
    increase the coefficients up to 2, but this can noticeably degrade the quality
    of samples. Negative values can be used to increase the likelihood of
    repetition.

    See also `presence_penalty` for penalizing tokens that have at least one
    appearance at a fixed rate.

    OpenAI compatible (follows OpenAI's conventions for handling token frequency and
    repetition penalties).

    Required range: `-2 <= x <= 2`
    """

    function_call: Optional[FunctionCall]
    """Deprecated in OpenAI.

    Use 'tool_choice' instead. This will be automatically transformed to
    tool_choice.
    """

    functions: Iterable[Function]
    """Deprecated in OpenAI.

    Use 'tools' instead. This will be automatically transformed to tools.
    """

    ignore_eos: bool
    """
    This setting controls whether the model should ignore the End of Sequence (EOS)
    token. When set to `True`, the model will continue generating tokens even after
    the EOS token is produced. By default, it stops when the EOS token is reached.
    """

    logit_bias: Optional[Dict[str, float]]
    """Modify the likelihood of specified tokens appearing in the completion.

    Accepts a json object that maps tokens (specified by their token ID in the
    tokenizer) to an associated bias value from -100 to 100. Mathematically, the
    bias is added to the logits generated by the model prior to sampling.
    """

    logprobs: Union[int, bool, None]
    """Include log probabilities in the response.

    This accepts either a boolean or an integer:

    If set to `true`, log probabilities are included and the number of alternatives
    can be controlled via `top_logprobs` (OpenAI-compatible behavior).

    If set to an integer N (0-5), include log probabilities for up to N most likely
    tokens per position in the legacy format.

    The API will always return the logprob of the sampled token, so there may be up
    to `logprobs+1` elements in the response when an integer is used. The maximum
    value for the integer form is 5.
    """

    max_completion_tokens: Optional[int]
    """Alias for max_tokens. Cannot be specified together with max_tokens."""

    max_tokens: Optional[int]
    """The maximum number of tokens to generate in the completion.

    If the token count of your prompt plus max_tokens exceeds the model's context
    length, the behavior depends on context_length_exceeded_behavior. By default,
    max_tokens will be lowered to fit in the context window instead of returning an
    error.
    """

    metadata: Optional[Dict[str, str]]
    """Additional metadata to store with the request for tracing/distillation."""

    min_p: Optional[float]
    """Minimum probability threshold for token selection.

    Only tokens with probability >= min_p are considered for selection. This is an
    alternative to `top_p` and `top_k` sampling.

    Required range: `0 <= x <= 1`
    """

    mirostat_lr: Optional[float]
    """
    Specifies the learning rate for the Mirostat sampling algorithm, which controls
    how quickly the model adjusts its token distribution to maintain the target
    perplexity. A smaller value slows down the adjustments, leading to more stable
    but gradual shifts, while higher values speed up corrections at the cost of
    potential instability.
    """

    mirostat_target: Optional[float]
    """Defines the target perplexity for the Mirostat algorithm.

    Perplexity measures the unpredictability of the generated text, with higher
    values encouraging more diverse and creative outputs, while lower values
    prioritize predictability and coherence. The algorithm dynamically adjusts the
    token selection to maintain this target during text generation.

    If not specified, Mirostat sampling is disabled.
    """

    n: int
    """How many completions to generate for each prompt.

    **Note:** Because this parameter generates many completions, it can quickly
    consume your token quota. Use carefully and ensure that you have reasonable
    settings for `max_tokens` and `stop`.

    Required range: `1 <= x <= 128`

    Example: `1`
    """

    parallel_tool_calls: Optional[bool]
    """Enable parallel function calling."""

    perf_metrics_in_response: Optional[bool]
    """Whether to include performance metrics in the response body.

    **Non-streaming requests:** Performance metrics are always included in response
    headers (e.g., `fireworks-prompt-tokens`,
    `fireworks-server-time-to-first-token`). Setting this to `true` additionally
    includes the same metrics in the response body under the `perf_metrics` field.

    **Streaming requests:** Performance metrics are only included in the response
    body under the `perf_metrics` field in the final chunk (when `finish_reason` is
    set). This is because headers may not be accessible during streaming.

    The response body `perf_metrics` field contains the following metrics:

    **Basic Metrics (all deployments):**

    - `prompt-tokens`: Number of tokens in the prompt
    - `server-time-to-first-token`: Time from request start to first token (in
      seconds)
    - `server-processing-time`: Total processing time (in seconds, only for
      completed requests)

    **Predicted Outputs Metrics:**

    - `speculation-prompt-tokens`: Number of speculative prompt tokens
    - `speculation-prompt-matched-tokens`: Number of matched speculative prompt
      tokens (for completed requests)

    **Dedicated Deployment Only Metrics:**

    - `speculation-generated-tokens`: Number of speculative generated tokens (for
      completed requests)
    - `speculation-acceptance`: Speculation acceptance rates by position
    - `cached-prompt-tokens`: Number of cached prompt tokens
    - `backend-host`: Hostname of the backend server
    - `num-concurrent-requests`: Number of concurrent requests
    - `deployment`: Deployment name
    - `tokenizer-queue-duration`: Time spent in tokenizer queue
    - `tokenizer-duration`: Time spent in tokenizer
    - `prefill-queue-duration`: Time spent in prefill queue
    - `prefill-duration`: Time spent in prefill
    - `generation-queue-duration`: Time spent in generation queue
    """

    prediction: Optional[Prediction]
    """OpenAI-compatible predicted output for speculative decoding.

    Can be a PredictedOutput object or a simple string. Automatically transformed to
    speculation.
    """

    presence_penalty: Optional[float]
    """Number between -2.0 and 2.0.

    Positive values penalize new tokens based on whether they appear in the text so
    far, increasing the model's likelihood to talk about new topics.

    Reasonable value is around 0.1 to 1 if the aim is to just reduce repetitive
    samples somewhat. If the aim is to strongly suppress repetition, then one can
    increase the coefficients up to 2, but this can noticeably degrade the quality
    of samples. Negative values can be used to increase the likelihood of
    repetition.

    See also `frequency_penalty` for penalizing tokens at an increasing rate
    depending on how often they appear.

    OpenAI compatible (follows OpenAI's conventions for handling token frequency and
    repetition penalties).

    Required range: `-2 <= x <= 2`
    """

    prompt_cache_isolation_key: Optional[str]
    """Isolation key for prompt caching to separate cache entries."""

    prompt_cache_max_len: Optional[int]
    """Maximum length of the prompt to cache."""

    prompt_truncate_len: Optional[int]
    """The size (in tokens) to which to truncate chat prompts.

    This includes the system prompt (if any), previous user/assistant messages, and
    the current user message. Earlier user/assistant messages will be evicted first
    to fit the prompt into this length. The system prompt is preserved whenever
    possible and only truncated as a last resort.

    This should usually be set to a number much smaller << than the model's maximum
    context size, to allow enough remaining tokens for generating a response.

    If omitted, you may receive "prompt too long" errors in your responses as
    conversations grow. Note that even with this set, you may still receive "prompt
    too long" errors if individual messages (such as a very long system prompt or
    user message) exceed the model's context window on their own.
    """

    raw_output: Optional[bool]
    """Return raw output from the model."""

    reasoning_effort: Union[Literal["low", "medium", "high", "none"], int, bool, None]
    """Controls reasoning behavior for supported models.

    When enabled, the model's reasoning appears in the `reasoning_content` field of
    the response, separate from the final answer in `content`.

    **Accepted values:**

    - **String** (OpenAI-compatible): `'low'`, `'medium'`, or `'high'` to enable
      reasoning with varying effort levels; `'none'` to disable reasoning.
    - **Boolean** (Fireworks extension): `true` to enable reasoning, `false` to
      disable it.
    - **Integer** (Fireworks extension): A positive integer to set a hard token
      limit on reasoning output (only effective for grammar-based reasoning models).

    **Important:** Boolean values are normalized internally: `true` becomes
    `'medium'`, and `false` becomes `'none'`. This normalization happens before
    model-specific validation, so if a model doesn't support `'none'`, passing
    `false` will produce an error referencing `'none'`.

    **Model-specific behavior:**

    - **Qwen3 (e.g., Qwen3-8B)**: Grammar-based reasoning. Default reasoning on. Use
      `'none'` or `false` to disable. Supports integer token limits to cap reasoning
      output. `'low'` maps to a default token limit (~3000 tokens).
    - **DeepSeek V3.1, DeepSeek V3.2**: Binary on/off reasoning. Default reasoning
      off. Any value except `'none'`/`false`/`null` enables reasoning; effort levels
      and integers have no additional effect.
    - **GLM 4.5, GLM 4.5 Air, GLM 4.6**: Binary on/off reasoning. Default reasoning
      on. Use `'none'` or `false` to disable; effort levels and integers have no
      additional effect.
    - **Harmony (OpenAI GPT-OSS 120B, GPT-OSS 20B)**: Accepts only `'low'`,
      `'medium'`, or `'high'`. Does not support `'none'`, `false`, or integer
      values—using these will return an error (e.g., "Invalid reasoning effort:
      none"). When omitted, defaults to `'medium'`. Lower effort produces faster
      responses with shorter reasoning.
    """

    repetition_penalty: Optional[float]
    """Applies a penalty to repeated tokens to discourage or encourage repetition.

    A value of `1.0` means no penalty, allowing free repetition. Values above `1.0`
    penalize repetition, reducing the likelihood of repeating tokens. Values between
    `0.0` and `1.0` reward repetition, increasing the chance of repeated tokens. For
    a good balance, a value of `1.2` is often recommended. Note that the penalty is
    applied to both the generated output and the prompt in decoder-only models.

    Required range: `0 <= x <= 2`
    """

    response_format: Optional[ResponseFormat]
    """Allows to force the model to produce specific output format.

    Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the
    message the model generates is valid JSON.

    If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g.,
    `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

    Important: when using JSON mode, it's crucial to also instruct the model to
    produce JSON via a system or user message. Without this, the model may generate
    an unending stream of whitespace until the generation reaches the token limit,
    resulting in a long-running and seemingly "stuck" request.

    Also note that the message content may be partially cut off if
    `finish_reason="length"`, which indicates the generation exceeded `max_tokens`
    or the conversation exceeded the max context length. In this case the return
    value might not be a valid JSON.
    """

    return_token_ids: Optional[bool]
    """Return token IDs alongside text to avoid retokenization drift."""

    seed: Optional[int]
    """Random seed for deterministic sampling."""

    speculation: Union[str, Iterable[int], None]
    """Speculative decoding prompt or token IDs to speed up generation."""

    stop: Union[str, SequenceNotStr[str], None]
    """Up to 4 sequences where the API will stop generating further tokens.

    The returned text will NOT contain the stop sequence.
    """

    temperature: Optional[float]
    """What sampling temperature to use, between 0 and 2.

    Higher values like 0.8 will make the output more random, while lower values like
    0.2 will make it more focused and deterministic.

    We generally recommend altering this or top_p but not both.

    Required range: `0 <= x <= 2`

    Example: `1`
    """

    tool_choice: ToolChoice
    """Controls which (if any) tool is called by the model.

    - `none`: the model will not call any tool and instead generates a message.
    - `auto`: the model can pick between generating a message or calling one or more
      tools.
    - `required` (alias: `any`): the model must call one or more tools. To force a
      specific function, pass an object of the form
      `{ "type": "function", "name": "my_function" }` or
      `{ "type": "function", "function": { "name": "my_function" } }` for OpenAI
      compatibility.
    """

    tools: Iterable[ChatCompletionTool]
    """A list of tools the model may call.

    Currently, only functions are supported as a tool.

    Use this to provide a list of functions the model may generate JSON inputs for.

    See the our
    [model library](https://app.fireworks.ai/models/?filter=LLM&functionCalling=true)
    for the list of supported models
    """

    top_k: Optional[int]
    """
    Top-k sampling is another sampling method where the k most probable next tokens
    are filtered and the probability mass is redistributed among only those k next
    tokens. The value of k controls the number of candidates for the next token at
    each step during text generation. Must be between 0 and 100.

    Required range: `0 <= x <= 100`

    Example: `50`
    """

    top_logprobs: Optional[int]
    """
    An integer between 0 and 5 specifying the number of most likely tokens to return
    at each token position, each with an associated log probability. The minimum
    value is 0 and the maximum value is 5.

    When `logprobs` is set, `top_logprobs` can be used to modify how many top log
    probabilities are returned. If `top_logprobs` is not set, the API will return up
    to `logprobs` tokens per position.

    Required range: `0 <= x <= 5`
    """

    top_p: Optional[float]
    """
    An alternative to sampling with temperature, called nucleus sampling, where the
    model considers the results of the tokens with top_p probability mass. So 0.1
    means only the tokens comprising the top 10% probability mass are considered.

    We generally recommend altering this or temperature but not both.

    Required range: `0 <= x <= 1`

    Example: `1`
    """

    typical_p: Optional[float]
    """Typical-p sampling is an alternative to nucleus sampling.

    It considers the most typical tokens whose cumulative probability is at most
    typical_p.

    Required range: `0 <= x <= 1`
    """

    user: Optional[str]
    """
    A unique identifier representing your end-user, which can help monitor and
    detect abuse.
    """


class FunctionCallFunctionNameSpec(TypedDict, total=False):
    name: Required[str]


FunctionCall: TypeAlias = Union[Literal["auto", "none"], FunctionCallFunctionNameSpec]


class Function(TypedDict, total=False):
    name: Required[str]
    """The name of the function to be called.

    Must be a-z, A-Z, 0-9, or contain underscores and dashes, with a maximum length
    of 64.
    """

    description: Optional[str]
    """
    A description of what the function does, used by the model to choose when and
    how to call the function.
    """

    parameters: Dict[str, object]
    """The parameters the function accepts, described as a JSON Schema object.

    The JSON Schema object should have the following structure:

    ```json
    {
      "type": "object",
      "required": ["param1", "param2"],
      "properties": {
        "param1": {
          "type": "string",
          "description": "..."
        },
        "param2": {
          "type": "number",
          "description": "..."
        }
      }
    }
    ```

    - The `type` field must be `"object"`.
    - The `required` field is an array of strings indicating which parameters are
      required.
    - The `properties` field is a map of property names to their definitions, where
      each property is an object with `type` (string) and `description` (string)
      fields.

    To describe a function that accepts no parameters, provide the value:

    ```json
    { "type": "object", "properties": {} }
    ```
    """

    strict: Optional[bool]


class PredictionPredictedOutputContentUnionMember1ImageURL(TypedDict, total=False):
    url: Required[str]

    detail: Optional[str]


class PredictionPredictedOutputContentUnionMember1(TypedDict, total=False):
    type: Required[str]

    image_url: Optional[PredictionPredictedOutputContentUnionMember1ImageURL]

    text: Optional[str]


class PredictionPredictedOutput(TypedDict, total=False):
    """OpenAI-compatible struct for the "speculation" field."""

    content: Required[Union[str, Iterable[PredictionPredictedOutputContentUnionMember1]]]

    type: Literal["content"]


Prediction: TypeAlias = Union[PredictionPredictedOutput, str]


class ResponseFormat(TypedDict, total=False):
    """Allows to force the model to produce specific output format.

    Setting to `{ "type": "json_object" }` enables JSON mode, which guarantees the message the model generates is valid JSON.

    If `"type"` is `"json_schema"`, a JSON schema must be provided. E.g., `response_format = {"type": "json_schema", "json_schema": <json_schema>}`.

    Important: when using JSON mode, it's crucial to also instruct the model to produce JSON via a system or user message. Without this, the model may generate an unending stream of whitespace until the generation reaches the token limit, resulting in a long-running and seemingly "stuck" request.

    Also note that the message content may be partially cut off if `finish_reason="length"`, which indicates the generation exceeded `max_tokens` or the conversation exceeded the max context length. In this case the return value might not be a valid JSON.
    """

    type: Required[Literal["json_object", "json_schema", "grammar", "text"]]

    grammar: Optional[str]

    json_schema: Union[str, Dict[str, object], None]

    schema: Union[str, Dict[str, object], None]


class ToolChoiceFunctionSelectionFunction(TypedDict, total=False):
    name: Required[str]


class ToolChoiceFunctionSelection(TypedDict, total=False):
    type: Required[Literal["function"]]

    function: Optional[ToolChoiceFunctionSelectionFunction]


ToolChoice: TypeAlias = Union[Literal["auto", "none", "any", "required"], ToolChoiceFunctionSelection]


class CompletionCreateParamsNonStreaming(CompletionCreateParamsBase, total=False):
    stream: Optional[Literal[False]]
    """Whether to stream back partial progress.

    If set, tokens will be sent as data-only
    [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
    as they become available, with the stream terminated by a `data: [DONE]`
    message.
    """


class CompletionCreateParamsStreaming(CompletionCreateParamsBase):
    stream: Required[Literal[True]]
    """Whether to stream back partial progress.

    If set, tokens will be sent as data-only
    [server-sent events](https://developer.mozilla.org/en-US/docs/Web/API/Server-sent_events/Using_server-sent_events#Event_stream_format)
    as they become available, with the stream terminated by a `data: [DONE]`
    message.
    """


CompletionCreateParams = Union[CompletionCreateParamsNonStreaming, CompletionCreateParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ModelGetUploadEndpointResponse"]


class ModelGetUploadEndpointResponse(BaseModel):
    filename_to_signed_urls: Optional[Dict[str, str]] = FieldInfo(alias="filenameToSignedUrls", default=None)

    filename_to_unsigned_uris: Optional[Dict[str, str]] = FieldInfo(alias="filenameToUnsignedUris", default=None)
    """Unsigned URIs (e.g.

    s3://bucket/key, gs://bucket/key) for uploading model files. Returned when the
    caller has permission to upload to the URIs.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..types import (
    batch_inference_job_get_params,
    batch_inference_job_list_params,
    batch_inference_job_create_params,
)
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.batch_inference_job import BatchInferenceJob
from ..types.batch_inference_job_list_response import BatchInferenceJobListResponse
from ..types.shared_params.inference_parameters import InferenceParameters

__all__ = ["BatchInferenceJobsResource", "AsyncBatchInferenceJobsResource"]


class BatchInferenceJobsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> BatchInferenceJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return BatchInferenceJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchInferenceJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return BatchInferenceJobsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str | None = None,
        batch_inference_job_id: str | Omit = omit,
        continued_from_job_name: str | Omit = omit,
        display_name: str | Omit = omit,
        inference_parameters: InferenceParameters | Omit = omit,
        input_dataset_id: str | Omit = omit,
        model: str | Omit = omit,
        output_dataset_id: str | Omit = omit,
        precision: Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchInferenceJob:
        """
        Create Batch Inference Job

        Args:
          batch_inference_job_id: ID of the batch inference job.

          continued_from_job_name: The resource name of the batch inference job that this job continues from. Used
              for lineage tracking to understand job continuation chains.

          inference_parameters: Parameters controlling the inference process.

          input_dataset_id: The name of the dataset used for inference. This is required, except when
              continued_from_job_name is specified.

          model: The name of the model to use for inference. This is required, except when
              continued_from_job_name is specified.

          output_dataset_id: The name of the dataset used for storing the results. This will also contain the
              error file.

          precision: The precision with which the model should be served. If PRECISION_UNSPECIFIED, a
              default will be chosen based on the model.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            f"/v1/accounts/{account_id}/batchInferenceJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs",
            body=maybe_transform(
                {
                    "continued_from_job_name": continued_from_job_name,
                    "display_name": display_name,
                    "inference_parameters": inference_parameters,
                    "input_dataset_id": input_dataset_id,
                    "model": model,
                    "output_dataset_id": output_dataset_id,
                    "precision": precision,
                },
                batch_inference_job_create_params.BatchInferenceJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"batch_inference_job_id": batch_inference_job_id},
                    batch_inference_job_create_params.BatchInferenceJobCreateParams,
                ),
            ),
            cast_to=BatchInferenceJob,
        )

    def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchInferenceJobListResponse:
        """
        List Batch Inference Jobs

        Args:
          filter: Only jobs satisfying the provided filter (if specified) will be returned. See
              https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "created_time".

          page_size: The maximum number of batch inference jobs to return. The maximum page_size is
              200, values above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListBatchInferenceJobs call. Provide this
              to retrieve the subsequent page. When paginating, all other parameters provided
              to ListBatchInferenceJobs must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            f"/v1/accounts/{account_id}/batchInferenceJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    batch_inference_job_list_params.BatchInferenceJobListParams,
                ),
            ),
            cast_to=BatchInferenceJobListResponse,
        )

    def delete(
        self,
        batch_inference_job_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Batch Inference Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not batch_inference_job_id:
            raise ValueError(
                f"Expected a non-empty value for `batch_inference_job_id` but received {batch_inference_job_id!r}"
            )
        return self._delete(
            f"/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    def get(
        self,
        batch_inference_job_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchInferenceJob:
        """
        Get Batch Inference Job

        Args:
          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not batch_inference_job_id:
            raise ValueError(
                f"Expected a non-empty value for `batch_inference_job_id` but received {batch_inference_job_id!r}"
            )
        return self._get(
            f"/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"read_mask": read_mask}, batch_inference_job_get_params.BatchInferenceJobGetParams
                ),
            ),
            cast_to=BatchInferenceJob,
        )


class AsyncBatchInferenceJobsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncBatchInferenceJobsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchInferenceJobsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchInferenceJobsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/fw-ai-external/python-sdk#with_streaming_response
        """
        return AsyncBatchInferenceJobsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str | None = None,
        batch_inference_job_id: str | Omit = omit,
        continued_from_job_name: str | Omit = omit,
        display_name: str | Omit = omit,
        inference_parameters: InferenceParameters | Omit = omit,
        input_dataset_id: str | Omit = omit,
        model: str | Omit = omit,
        output_dataset_id: str | Omit = omit,
        precision: Literal[
            "PRECISION_UNSPECIFIED",
            "FP16",
            "FP8",
            "FP8_MM",
            "FP8_AR",
            "FP8_MM_KV_ATTN",
            "FP8_KV",
            "FP8_MM_V2",
            "FP8_V2",
            "FP8_MM_KV_ATTN_V2",
            "NF4",
            "FP4",
            "BF16",
            "FP4_BLOCKSCALED_MM",
            "FP4_MX_MOE",
        ]
        | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchInferenceJob:
        """
        Create Batch Inference Job

        Args:
          batch_inference_job_id: ID of the batch inference job.

          continued_from_job_name: The resource name of the batch inference job that this job continues from. Used
              for lineage tracking to understand job continuation chains.

          inference_parameters: Parameters controlling the inference process.

          input_dataset_id: The name of the dataset used for inference. This is required, except when
              continued_from_job_name is specified.

          model: The name of the model to use for inference. This is required, except when
              continued_from_job_name is specified.

          output_dataset_id: The name of the dataset used for storing the results. This will also contain the
              error file.

          precision: The precision with which the model should be served. If PRECISION_UNSPECIFIED, a
              default will be chosen based on the model.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            f"/v1/accounts/{account_id}/batchInferenceJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs",
            body=await async_maybe_transform(
                {
                    "continued_from_job_name": continued_from_job_name,
                    "display_name": display_name,
                    "inference_parameters": inference_parameters,
                    "input_dataset_id": input_dataset_id,
                    "model": model,
                    "output_dataset_id": output_dataset_id,
                    "precision": precision,
                },
                batch_inference_job_create_params.BatchInferenceJobCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"batch_inference_job_id": batch_inference_job_id},
                    batch_inference_job_create_params.BatchInferenceJobCreateParams,
                ),
            ),
            cast_to=BatchInferenceJob,
        )

    async def list(
        self,
        *,
        account_id: str | None = None,
        filter: str | Omit = omit,
        order_by: str | Omit = omit,
        page_size: int | Omit = omit,
        page_token: str | Omit = omit,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchInferenceJobListResponse:
        """
        List Batch Inference Jobs

        Args:
          filter: Only jobs satisfying the provided filter (if specified) will be returned. See
              https://google.aip.dev/160 for the filter grammar.

          order_by: A comma-separated list of fields to order by. e.g. "foo,bar" The default sort
              order is ascending. To specify a descending order for a field, append a " desc"
              suffix. e.g. "foo desc,bar" Subfields are specified with a "." character. e.g.
              "foo.bar" If not specified, the default order is by "created_time".

          page_size: The maximum number of batch inference jobs to return. The maximum page_size is
              200, values above 200 will be coerced to 200. If unspecified, the default is 50.

          page_token: A page token, received from a previous ListBatchInferenceJobs call. Provide this
              to retrieve the subsequent page. When paginating, all other parameters provided
              to ListBatchInferenceJobs must match the call that provided the page token.

          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            f"/v1/accounts/{account_id}/batchInferenceJobs"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "filter": filter,
                        "order_by": order_by,
                        "page_size": page_size,
                        "page_token": page_token,
                        "read_mask": read_mask,
                    },
                    batch_inference_job_list_params.BatchInferenceJobListParams,
                ),
            ),
            cast_to=BatchInferenceJobListResponse,
        )

    async def delete(
        self,
        batch_inference_job_id: str,
        *,
        account_id: str | None = None,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Batch Inference Job

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not batch_inference_job_id:
            raise ValueError(
                f"Expected a non-empty value for `batch_inference_job_id` but received {batch_inference_job_id!r}"
            )
        return await self._delete(
            f"/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )

    async def get(
        self,
        batch_inference_job_id: str,
        *,
        account_id: str | None = None,
        read_mask: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> BatchInferenceJob:
        """
        Get Batch Inference Job

        Args:
          read_mask: The fields to be returned in the response. If empty or "\\**", all fields will be
              returned.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id is None:
            account_id = self._client._get_account_id_path_param()
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not batch_inference_job_id:
            raise ValueError(
                f"Expected a non-empty value for `batch_inference_job_id` but received {batch_inference_job_id!r}"
            )
        return await self._get(
            f"/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}"
            if self._client._base_url_overridden
            else f"https://api.fireworks.ai/v1/accounts/{account_id}/batchInferenceJobs/{batch_inference_job_id}",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"read_mask": read_mask}, batch_inference_job_get_params.BatchInferenceJobGetParams
                ),
            ),
            cast_to=BatchInferenceJob,
        )


class BatchInferenceJobsResourceWithRawResponse:
    def __init__(self, batch_inference_jobs: BatchInferenceJobsResource) -> None:
        self._batch_inference_jobs = batch_inference_jobs

        self.create = to_raw_response_wrapper(
            batch_inference_jobs.create,
        )
        self.list = to_raw_response_wrapper(
            batch_inference_jobs.list,
        )
        self.delete = to_raw_response_wrapper(
            batch_inference_jobs.delete,
        )
        self.get = to_raw_response_wrapper(
            batch_inference_jobs.get,
        )


class AsyncBatchInferenceJobsResourceWithRawResponse:
    def __init__(self, batch_inference_jobs: AsyncBatchInferenceJobsResource) -> None:
        self._batch_inference_jobs = batch_inference_jobs

        self.create = async_to_raw_response_wrapper(
            batch_inference_jobs.create,
        )
        self.list = async_to_raw_response_wrapper(
            batch_inference_jobs.list,
        )
        self.delete = async_to_raw_response_wrapper(
            batch_inference_jobs.delete,
        )
        self.get = async_to_raw_response_wrapper(
            batch_inference_jobs.get,
        )


class BatchInferenceJobsResourceWithStreamingResponse:
    def __init__(self, batch_inference_jobs: BatchInferenceJobsResource) -> None:
        self._batch_inference_jobs = batch_inference_jobs

        self.create = to_streamed_response_wrapper(
            batch_inference_jobs.create,
        )
        self.list = to_streamed_response_wrapper(
            batch_inference_jobs.list,
        )
        self.delete = to_streamed_response_wrapper(
            batch_inference_jobs.delete,
        )
        self.get = to_streamed_response_wrapper(
            batch_inference_jobs.get,
        )


class AsyncBatchInferenceJobsResourceWithStreamingResponse:
    def __init__(self, batch_inference_jobs: AsyncBatchInferenceJobsResource) -> None:
        self._batch_inference_jobs = batch_inference_jobs

        self.create = async_to_streamed_response_wrapper(
            batch_inference_jobs.create,
        )
        self.list = async_to_streamed_response_wrapper(
            batch_inference_jobs.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            batch_inference_jobs.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            batch_inference_jobs.get,
        )
